/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton.org

Copyright (c) 2002, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>

void msg_error(char *str, char *filename, int error);
static int ProcessFile(char *filename, void *ptr);
static void translate_and_save(char *buffer,long len, char *filename);

int main(int argc, char **argv)
{
	int count;
	char *filename;
	

	if( argc < 2 )
	{
		fprintf(stderr,"Usage: %s <files>\n",argv[0]);
		exit(1);
	}

	count = 1;

	/*
	if( !strcmp(argv[1], "-v") )
	{
		verbose = 1;
		count++;
		if( argc < 3 )
		{
			fprintf(stderr,"Usage: %s <files>\n",argv[0]);
			exit(1);
		}
	}
	*/
start:
	{
		filename = argv[count];
		(void)ProcessFile( filename,NULL );

		count++;
	}
	if( count < argc )
		goto start;
	

	return 0;
}

static int ProcessFile(char *filename, void *ptr)
{
	FILE *file = NULL;
	char *buffer = NULL;
	long len;
	struct stat stat_rec;

	if( stat(filename, &stat_rec) )
	{
		msg_error("Error from stat",filename,errno);
		goto error;
	}
	if( S_ISDIR( stat_rec.st_mode ) )
	{
		fprintf(stderr,"%s is a directory, skipping\n",filename);
		goto error;
	}


	file = fopen( filename, "rb" );
	if( !file )
	{
		msg_error( "Error opening file for reading",filename,errno);
		goto error;
	}

	if( fseek(file,0,SEEK_END) )
	{
		msg_error( "Error seeking",filename,errno);
		goto error;
	}

	len = ftell(file);
	if( len == EOF )
	{
		msg_error( "Error getting file position",filename,errno);
		goto error;
	}
	
	if( fseek(file,0,SEEK_SET) )
	{
		msg_error("Error seeking",filename,errno);
		goto error;
	}

	buffer = (char *)malloc( sizeof(char) * len );
	if( !buffer )
	{
		msg_error("Couldn't get buffer",filename,errno);
		goto error;
	}
	
	if( fread( (void *)buffer, len, 1, file ) != 1 )
	{
		msg_error("Error reading",filename,errno);
		goto error;
	}

	// close the file for reading so we can open it for
	// writing.
	if( fclose(file) )
		msg_error( "Error closing file",filename,errno);
	file = NULL;
		
	// NOTE: I think it's safe to write over the file
	// because the file will never get longer,
	// it will either get shorter or stay the same
	// length.
	translate_and_save(buffer,len,filename);

error:
	// free the buffer.
	if( buffer )
		free( buffer );
	buffer = NULL;
	// close the file if needed
	if( file )
		(void)fclose(file);
	file = NULL;


	return 0;
}

static void translate_and_save(char *buffer,long len, char *filename)
{
	char *start, *ptr = buffer;
	long count, runLen;
	FILE *file = NULL;

	file = fopen(filename,"wb");
	if( !file )
	{
		msg_error("Error opening for writing",filename,errno);
		return;
	}

// we only deal with unix newlines, convert it to
// Unix linefeeds by running the newline util.
#define kNewLine 10
#define kMaxLen 80 // max characters per line

	start = ptr;
	count = 0;
	runLen = 0;
	while( count < len )
	{
		if( *ptr == kNewLine || (*ptr == ' ' && runLen >= kMaxLen) )
		{
			// write out the length of the run, and a newline.
			if( runLen )
			{
				if( fwrite( start, 1, runLen, file ) != runLen )
				{
					msg_error("Error writing to file",filename,errno);
					return;
				}
			}
			if( putc(kNewLine,file) == EOF )
			{
				msg_error("Error writing to file",filename,errno);
				return;
			}
			ptr++;
			start = ptr;
			runLen = 0;
		}
		else
		{
			ptr++;
			runLen++;
		}
		
		count++;
	}
	// if we have anything cached up, write it out.
	if( runLen )
	{
		if( fwrite( start, 1, runLen, file ) != runLen )
		{
			msg_error("Error writing to file",filename,errno);
			return;
		}
	}
	
}

void msg_error(char *str, char *filename, int error)
{
	fprintf(stderr,"%s: %s, %s\n",
		str,
		filename,
		strerror(error)
		);
}



